@@ -99,9 +99,11 @@ def spreadsheet(screen_width,screen_height,file=None):
                         try:
                             return self.headers[section]  # column headers maybe out of range
                         except :
-                            return self.formatNumericHeader(str(section)) # return number instead if out of range
+                            return str(section)
+                            #return self.formatNumericHeader(str(section)) # return number instead if out of range
                     else:
-                        return self.formatNumericHeader(str(section)) # column
+                        return str(section)
+                        #return self.formatNumericHeader(str(section)) # column
                 else:
                     return str(section)  # row
 
@@ -329,6 +331,7 @@ def spreadsheet(screen_width,screen_height,file=None):
         global saved_file
         filter = "Pandas Object(*.pdobj);;Numpy Object(*.npobj);;CSV(*.csv);;JSON(*.json);;HTML(*.html);;Excel(*.xlsx);;PDF( *.pdf)"
         if saveAs:
+            global current_file_name
             directory, filter = QFileDialog.getSaveFileName(menuWidget, 'Save File',filter=filter)
 
         elif directory==None and current_file_name == None:
@@ -342,8 +345,6 @@ def spreadsheet(screen_width,screen_height,file=None):
             else:
                 filter = '.pdobj'
 
-        
-
         elif current_file_name !=None:
             directory= current_file_name
             filter = '.'+directory.split('.')[-1]
@@ -356,14 +357,20 @@ def spreadsheet(screen_width,screen_height,file=None):
         if '.pdobj' in filter:
             directory = directory+'.pdobj' if '.pdobj' not in directory else directory
             joblib.dump(data,directory,9)
+            if saveAs:
+                current_file_name = directory
 
         elif '.csv' in filter:
             directory = directory+'.csv' if '.csv' not in directory else directory
             data.to_csv(directory, index=False,header=False)
+            if saveAs:
+                current_file_name = directory
 
         elif '.json' in filter:
             directory = directory+'.json' if '.json' not in directory else directory
             out = data.to_json(directory,index=False,orient= 'table')
+            if saveAs:
+                current_file_name = directory
 
         elif '.html' in filter:
             directory = directory+'.html' if '.html' not in directory else directory
@@ -371,19 +378,29 @@ def spreadsheet(screen_width,screen_height,file=None):
             with open(directory,'w') as f:
                 f.write(html)
                 f.close()
+            if saveAs:
+                current_file_name = directory
 
         elif'.xlsx'in directory:
             directory = directory+'.xlsx' if '.xlsx' not in directory else directory
             data.to_excel(directory,index=False)
+            if saveAs:
+                current_file_name = directory
 
         elif '.h5'in filter:
+            directory = directory+'.h5' if '.h5' not in directory else directory
             data.to_hdf(directory,key='0')
+            if saveAs:
+                current_file_name = directory
 
         else :
             print('file type not supported yet')
             alertbox('file type not supported yet\nrequest feature on:\nhttps://github.com/YC-Lammy/np_spreadsheet/issues')
+            return None
         saved_file = True
 
+        return directory
+
     def exportJoblib(dtype='np'): # export numpy array by joblib
 
         filename, filter = QFileDialog.getSaveFileName(menuWidget, 'Save File')
@@ -453,6 +470,7 @@ def spreadsheet(screen_width,screen_height,file=None):
         rowCount.setRange(row, row +10000)
         rowCount.setValue(row)
         dtypeLabel2.setText(tableWidget.model().array.dtype.name)
+        shapeLabel.setText('shape: '+str(tableWidget.model().array.shape))
 
     def spreadsheetCommand(interactive=False,scripting = False): # call function from spreadsheet command.py
         global saved_file
@@ -646,7 +664,123 @@ def spreadsheet(screen_width,screen_height,file=None):
 
     ######################## manage Pip #####################################
     def managePip():
-        pass
+        from bs4 import BeautifulSoup
+        from urllib.parse import urljoin
+        import requests
+        import re
+        import pkg_resources
+        from subprocess import Popen, PIPE
+
+        def searchPip(query: str):
+            api_url = 'https://pypi.org/search/'
+            snippets = []
+            s = requests.Session()
+            for page in range(1, 3):
+                params = {'q': query, 'page': page}
+                r = s.get(api_url, params=params)
+                soup = BeautifulSoup(r.text, 'html.parser')
+                snippets += soup.select('a[class*="snippet"]')
+                if not hasattr(s, 'start_url'):
+                    s.start_url = r.url.rsplit('&page', maxsplit=1).pop(0)
+            table = []
+            for snippet in snippets:
+                link = urljoin(api_url, snippet.get('href'))
+                package = re.sub(r"\s+", " ", snippet.select_one('span[class*="name"]').text.strip())
+                version = re.sub(r"\s+", " ", snippet.select_one('span[class*="version"]').text.strip())
+                released = re.sub(r"\s+", " ", snippet.select_one('span[class*="released"]').text.strip())
+                description = re.sub(r"\s+", " ", snippet.select_one('p[class*="description"]').text.strip())
+                emoji = ':open_file_folder:'
+                table.append([package,version,released,description])
+
+            return table
+
+
+        installed_packages = pkg_resources.working_set
+        listPK = []
+        installedPK = []
+        for i in installed_packages:
+            listPK.append([i.key,i.version])
+        scroll = QScrollArea()
+        mainWidget = QWidget()
+        mainlayout = QVBoxLayout()
+
+        searchbar = QLineEdit()
+        searchbar.setPlaceholderText('type and press enter to search for new packages')
+        searchbar.returnPressed.connect(lambda:print(searchbar.text()))
+        mainlayout.addWidget(searchbar)
+        mainlayout.addWidget(scroll)
+
+
+        box = QDialog()
+        box.setMinimumSize(screen_width/2,screen_height/2)
+        box.setWindowTitle('Analyze data')
+
+        
+
+        layout = QGridLayout(mainWidget)
+
+        num = 0
+        def pip_upgrade(package):
+            p = Popen([sys.executable,'-m','pip','install','--upgrade',package], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            output, err = p.communicate()
+
+            alert = QMessageBox()
+            alert.setWindowTitle('Pip output')
+            alert.setWindowIcon(mainWidget.style().standardIcon(QStyle.SP_MessageBoxInformation))
+            alert.setText(str(output)[2:-1]+'\r\n'+str(err))
+            alert.setAttribute(Qt.WA_DeleteOnClose) # prevent memory leak
+            alert.setTextInteractionFlags(Qt.TextBrowserInteraction)
+            alert.exec()
+
+        def pip_uninstall(package):
+            p = Popen([sys.executable,'-m','pip','uninstall',package,'-y'], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            output, err = p.communicate()
+
+            alert = QMessageBox()
+            alert.setWindowTitle('Pip output')
+            alert.setWindowIcon(QIcon(os.path.join(pic_file_path,'warning.png')))
+            alert.setText(str(output)[2:-1]+'\r\n'+str(err))
+            alert.setAttribute(Qt.WA_DeleteOnClose) # prevent memory leak
+            alert.setTextInteractionFlags(Qt.TextBrowserInteraction)
+            alert.exec()
+
+        def pip_install(package):
+            p = Popen([sys.executable,'-m','pip','install',package], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            output, err = p.communicate()
+
+            alert = QMessageBox()
+            alert.setWindowTitle('Pip output')
+            alert.setWindowIcon(QIcon(os.path.join(pic_file_path,'warning.png')))
+            alert.setText(str(output)[2:-1]+'\r\n'+str(err))
+            alert.setAttribute(Qt.WA_DeleteOnClose) # prevent memory leak
+            alert.setTextInteractionFlags(Qt.TextBrowserInteraction)
+            alert.exec()
+
+        for i in sorted(listPK):
+            layout.addWidget(QLabel(i[0]+'=='+i[1]),num,0)
+            ub= QPushButton('upgrade')
+            
+            # force free variable i into a fixed statment
+            exec(f"ub.clicked.connect(lambda:pip_upgrade('{i[0]}'))",locals())
+            
+            ub2 = QPushButton('uninstall')
+
+            exec(f"ub2.clicked.connect(lambda:pip_uninstall('{i[0]}'))",locals())
+            
+            layout.addWidget(ub,num,1)
+            layout.addWidget(ub2,num,2)
+            num+=1
+
+
+        box.setLayout(mainlayout)
+
+        scroll.setWidget(mainWidget)
+        scroll.setWidgetResizable(True)
+
+        box.setWindowModality(Qt.WindowModal)
+        box.setAttribute(Qt.WA_DeleteOnClose)
+        box.exec_()
+
 
 ##############################################################################################################################################
 ############################ set up ##########################################################################################################
@@ -729,6 +863,31 @@ def spreadsheet(screen_width,screen_height,file=None):
     menuLayout_home = QVBoxLayout()
     menuLayout_home.setAlignment(Qt.AlignTop)
 
+
+    def numpyFn(func:str):
+        try:
+            if func == 'reshape':
+                tableWidget.model().array = tableWidget.model().array.reshape()
+                tableWidget.update()
+                updateInfo()
+
+        except Exception as e:
+            alertbox(e)
+
+    numpyBt = QToolButton()
+    numpyBt.setText('numpy')
+    numpyMenu = QMenu(numpyBt)
+
+    numpyMenu.addAction('reshape')
+    numpyMenu.addAction('resize')
+    numpyMenu.addAction('transpose')
+    numpyMenu.addAction('arround')
+    numpyMenu.addAction('slicing')
+    numpyMenu.addAction('ravel')
+
+    numpyBt.setMenu(numpyMenu)
+    numpyBt.setPopupMode(QToolButton.InstantPopup)
+
     def resizeTableToContent():
         tableWidget.resizeRowsToContents()
         tableWidget.resizeColumnsToContents()
@@ -758,6 +917,15 @@ def spreadsheet(screen_width,screen_height,file=None):
         tableWidget.update()
         changeSettings('fontsize',fontSizeCB.currentText())
 
+    def printFile():
+        if saved_file == False or current_file_name == None:
+            dir = saveFile()
+        else:
+            dir = current_file_name
+
+        i = sys.platform
+        
+
     fontSizeCB.currentIndexChanged.connect(setFontSize)
 
     dtypeLabel = QLabel('dtype:')
@@ -777,22 +945,42 @@ def spreadsheet(screen_width,screen_height,file=None):
     columnCount.setReadOnly(True)
     columnCount.setFixedSize(100,30)
 
+    shapeLabel = QLabel('shape: '+str(tableWidget.model().array.shape))
+
+########################################## toll bar1 #################################################################
     bar1 = QToolBar()
     bar1.setIconSize(QSize(int(screen_height/30),int(screen_height/30)))
     bar1.addAction(QIcon(os.path.join(pic_file_path,'save.png')),'Save file').triggered.connect(saveFile)
     bar1.addAction(QIcon(os.path.join(pic_file_path,'file.png')),'Open file').triggered.connect(pick_sys_file)
     bar1.addAction(QIcon(os.path.join(pic_file_path,'newFile.png')),'New file')
+
     bar1.addSeparator()
+
     bar1.addAction(QIcon(os.path.join(pic_file_path,'exportCSV.png')),'Export csv')
     bar1.addAction(QIcon(os.path.join(pic_file_path,'printer.png')),'Print')
+
     bar1.addSeparator()
+
     bar1.addAction(QIcon(os.path.join(pic_file_path,'cut_icon.png')),'Cut')
-    bar1.addAction(QIcon(),'Copy')
-    bar1.addAction(QIcon(),'Paste')
+    bar1.addAction('Copy')#.triggered.connect(cutCopyPasteHandler)
+    bar1.addAction('Paste')
+
     bar1.addSeparator()
+
+    bar1.addAction('undo')
+    bar1.addAction('redo')
+    bar1.addSeparator()
+
+    bar1.addWidget(numpyBt)
+    bar1.addAction('reshape')
+    bar1.addAction('astype')
+
+    bar1.addSeparator()
+
     bar1.addAction(QIcon(os.path.join(pic_file_path,'cellResize.png')),'resize cell to content').triggered.connect(resizeTableToContent)
     menuLayout_home.addWidget(bar1)
 
+################################# tool bar2 ###############################################################################################
     bar2 = QToolBar()
     bar2.setIconSize(QSize(int(screen_height/30),int(screen_height/30)))
     bar2.addWidget(fontTypeCB)
@@ -800,10 +988,11 @@ def spreadsheet(screen_width,screen_height,file=None):
     bar2.addWidget(dtypeLabel)
     bar2.addWidget(dtypeLabel2)
     bar2.addSeparator()
-    bar2.addWidget(rowLabel)
-    bar2.addWidget(rowCount)
-    bar2.addWidget(columnLabel)
-    bar2.addWidget(columnCount)
+    bar2.addWidget(shapeLabel)
+    #bar2.addWidget(rowLabel)
+    #bar2.addWidget(rowCount)
+    #bar2.addWidget(columnLabel)
+    #bar2.addWidget(columnCount)
     menuLayout_home.addWidget(bar2)
 
     homeWidget = QWidget()
@@ -812,11 +1001,19 @@ def spreadsheet(screen_width,screen_height,file=None):
     menuLayout.addWidget(homeWidget)
 
 ##########################################################################
+
     table_tab_box.addWidget(menuWidget)
     table_tab_box.addWidget(tableWidget)
 
 ########################### Menu bar ###############################################
+    
+    # the menu bar on mac os is absolutly disaster, menu that are not recognized by the os will not be rendered
+    # the solution is to also put it into the tool bar if sys.platform == 'darwin'(to do)
+
     bar = QMenuBar()
+
+    #bar.setNativeMenuBar(False)     # f**king prick mac os won't render the menu bar if not native
+
     menuLayout.setMenuBar(bar)
     bar.setGeometry(0, 0, int(menuWidget.frameGeometry().width() / 1.6), int(screen_height / 10))
 
@@ -997,7 +1194,7 @@ def main():
 
 ######################### handle sys argvs ###########################################################################################################
 
-    print(sys.argv)
+    print('argvs:'+str(sys.argv))
 
     if '-help' in sys.argv or '--help' in sys.argv or 'help' in sys.argv:
         print("""Usage: py-office-sheet [<option>...]
@@ -1054,10 +1251,13 @@ for more information, visit https://github.com/YC-Lammy/py-office-sheet
                 event.ignore() # when user choose no, stop exit event
 
 
-    app = QApplication(['-style fusion']+sys.argv)
+    app = QApplication(sys.argv)
+    size = app.primaryScreen().size()
+    screen_height = size.height()
+    screen_width = size.width()
 
     mainWidget = QWidget()
-    mainWidget.setLayout(spreadsheet(1920,1080,file=file)) # spreedsheet returns a layout
+    mainWidget.setLayout(spreadsheet(screen_width,screen_height,file=file)) # spreedsheet returns a layout
     mainWidget.show()
     mainWidget.closeEvent = closeEventHandler # reassign the app's close event
     mainWidget.setWindowState(Qt.WindowMaximized)
